unit Main;

(* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *  Copyright (C) 2005 - 2006 Martin Offenwanger                             *
 *  Mail: coder@dsplayer.de                                                  *
 *  Web:  http://www.dsplayer.de                                             *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *)
{
@author(Martin Offenwanger: coder@dsplayer.de)
@created(Apr 03, 2005)
@lastmod(Jun 27, 2006)
}

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, DirectShow9, ActiveX, StdCtrls, DSUtil, ExtCtrls, BTAsyncExTypes,
  Buttons, TabNotBk, ShellAPI, ShlObj, ComCtrls, XPMan;

type
  TForm1 = class(TForm)
    OpenDialog1: TOpenDialog;
    Button1: TBitBtn;
    Button2: TBitBtn;
    Button4: TBitBtn;
    Button3: TBitBtn;
    GroupBox1: TGroupBox;
    RadioButton1: TRadioButton;
    RadioButton2: TRadioButton;
    RadioButton5: TRadioButton;
    procedure Button1Click(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure Button2Click(Sender: TObject);
    procedure Button3Click(Sender: TObject);
    procedure Button4Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    m_AsyncExDir: string;
    m_GraphBuilder: IGraphBuilder;
    m_MediaControl: IMediaControl;
    m_FileSource: IFilesourcefilter;
    m_BTAsyncExControl: IBTAsyncExControl;
    m_BTAsyncEx: IBaseFilter;
    m_Mpeg1Splitter: IBaseFilter;
    m_Pin: IPin;
    m_FileStream: TFileStream;
    m_MemoryStream: TMemoryStream;
    procedure NilAll();
    procedure OpenFile(f_file: string);
    procedure OpenfileTFilestream(f_file: string);
    procedure OpenfileTMemorystream(f_file: string);
        // helper function (used in OpenfileTMemorystream)
    function IsFileSizeValid(f_filename: string; f_maxSize: LongInt): boolean;
  public
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

procedure TForm1.NilAll();
begin
  // required before destroying filter and interface (important!)
  if Assigned(m_MediaControl) then
    m_MediaControl.Stop;
  // Cleanup Filter and it's interfaces
  if Assigned(m_BTAsyncEx) then
    m_BTAsyncEx := nil;
  if Assigned(m_BTAsyncExControl) then
    m_BTAsyncExControl := nil;
  if Assigned(m_Pin) then
    m_Pin := nil;
  if Assigned(m_FileSource) then
    m_FileSource := nil;
  if Assigned(m_MediaControl) then
    m_MediaControl := nil;
  // destroy all streams
  if m_FileStream <> nil then begin
    m_FileStream.Free;
    m_FileStream := nil;
  end;
  if m_MemoryStream <> nil then begin
    m_MemoryStream.Free;
    m_MemoryStream := nil;
  end;
//  BTCoFreeFileInstance(m_BTAsyncEx);
  if Assigned(m_GraphBuilder) then
    m_GraphBuilder := nil;
end;

procedure TForm1.OpenfileTFilestream(f_file: string);
begin
  m_FileStream := TFileStream.Create(f_file, fmOpenRead or fmShareDenyWrite);
  CheckDSError(CoCreateInstance(TGUID(CLSID_FilterGraph), nil, CLSCTX_INPROC,
    TGUID(IID_IGraphBuilder), m_GraphBuilder));
  CheckDSError(m_GraphBuilder.QueryInterface(IID_IMediaControl, m_MediaControl));
  CheckDSError(CoCreateInstance(CLSID_BTAsyncEx, nil, CLSCTX_INPROC,
    IID_IBaseFilter, m_BTAsyncEx));
  CheckDSError(m_BTAsyncEx.QueryInterface(IID_IBTAsyncExControl,
    m_BTAsyncExControl));
  CheckDSError(m_BTAsyncExControl.BTSetLoadFromStream(TStreamAdapter.Create(m_FileStream), m_FileStream.Size));
  CheckDSError(m_BTAsyncEx.FindPin(BTAsyncExPinID, m_Pin));
  CheckDSError(m_GraphBuilder.AddFilter(m_BTAsyncEx,
    StringToOleStr('DSPlayer Async FileSource')));
  CheckDSError(m_GraphBuilder.Render(m_Pin));
  CheckDSError(m_MediaControl.Run);
end;

procedure TForm1.OpenfileTMemorystream(f_file: string);
begin
  if not IsFileSizeValid(f_file, 100000000) then begin
    showmessage('In TMemorystream mode, the complete file will be loaded'#13#10 +
      'into the working memory. max supported TMemorystream  size: 100MB'#13#10#13#10 +
      'Your selected file is too big.... Please Open an other file');
    exit;
  end;
  m_MemoryStream := TMemoryStream.Create();
  m_MemoryStream.LoadFromFile(f_file);
  CheckDSError(CoCreateInstance(TGUID(CLSID_FilterGraph), nil, CLSCTX_INPROC,
    TGUID(IID_IGraphBuilder), m_GraphBuilder));
  CheckDSError(m_GraphBuilder.QueryInterface(IID_IMediaControl, m_MediaControl));
  CheckDSError(CoCreateInstance(CLSID_BTAsyncEx, nil, CLSCTX_INPROC,
    IID_IBaseFilter, m_BTAsyncEx));
  CheckDSError(m_BTAsyncEx.QueryInterface(IID_IBTAsyncExControl,
    m_BTAsyncExControl));
  CheckDSError(m_BTAsyncExControl.BTSetLoadFromStream(TStreamAdapter.Create(m_MemoryStream), m_MemoryStream.Size));
  CheckDSError(m_BTAsyncEx.FindPin(BTAsyncExPinID, m_Pin));
  CheckDSError(m_GraphBuilder.AddFilter(m_BTAsyncEx,
    StringToOleStr('DSPlayer Async FileSource')));
  CheckDSError(m_GraphBuilder.Render(m_Pin));
  CheckDSError(m_MediaControl.Run);
end;

procedure TForm1.OpenFile(f_file: string);
begin
  CheckDSError(CoCreateInstance(TGUID(CLSID_FilterGraph), nil, CLSCTX_INPROC,
    TGUID(IID_IGraphBuilder), m_GraphBuilder));
  CheckDSError(m_GraphBuilder.QueryInterface(IID_IMediaControl, m_MediaControl));
  CheckDSError(CoCreateInstance(CLSID_BTAsyncEx, nil, CLSCTX_INPROC,
    IID_IBaseFilter, m_BTAsyncEx));
  CheckDSError(m_BTAsyncEx.QueryInterface(IID_IFilesourcefilter,
    m_FileSource));
  CheckDSError(m_FileSource.Load(StringToOleStr(OpenDialog1.FileName), nil));
  CheckDSError(m_BTAsyncEx.FindPin(BTAsyncExPinID, m_Pin));
  CheckDSError(m_GraphBuilder.AddFilter(m_BTAsyncEx,
    StringToOleStr('DSPlayer Async FileSource')));
  CheckDSError(m_GraphBuilder.Render(m_Pin));
  CheckDSError(m_MediaControl.Run);
end;

procedure TForm1.Button1Click(Sender: TObject);
begin
  if not OpenDialog1.Execute then exit;
  NILALL;
  if RadioButton1.Checked then
    OpenFile(OpenDialog1.FileName);
  if RadioButton2.Checked then
    OpenfileTFilestream(OpenDialog1.FileName);
  if RadioButton5.Checked then
    OpenfileTMemorystream(OpenDialog1.FileName);
end;

procedure TForm1.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  NilAll;
  CoUninitialize;
end;

procedure TForm1.Button2Click(Sender: TObject);
begin
  if Assigned(m_MediaControl) then CheckDSError(m_MediaControl.Run);
end;

procedure TForm1.Button3Click(Sender: TObject);
begin
  if Assigned(m_MediaControl) then CheckDSError(m_MediaControl.Pause);
end;

procedure TForm1.Button4Click(Sender: TObject);
begin
  if Assigned(m_MediaControl) then CheckDSError(m_MediaControl.Stop);
end;

procedure TForm1.FormCreate(Sender: TObject);
var
  ResBin: Pointer;
  ResBinSize: Cardinal;
begin
  m_FileStream := nil;
  m_MemoryStream := nil;
end;

function TForm1.IsFileSizeValid(f_filename: string; f_maxSize: LongInt): boolean;
var
  l_file: file of Byte;
  l_size: LongInt;
begin
  l_size := 0;
  AssignFile(l_file, f_filename);
  Reset(l_file);
  try
    l_size := FileSize(l_file);
  finally
    CloseFile(l_file);
  end;
  if (l_size < f_maxSize) and (l_size > 0) then
    Result := true
  else
    result := false;
end;

end.

